from typing import Callable
from unittest.mock import AsyncMock, MagicMock


class DictObject(dict):
    def __getattr__(self, item):
        return self[item]

    def __setattr__(self, key, value):
        self[key] = value

    def delete(self):
        return


class Dictable(DictObject):
    """We need to mock a kelvin user object, which has an as_dict method"""

    def as_dict(self):
        return self

    def dict(self):
        return self

    async def save(self):
        return

    async def delete(self):
        return


class KelvinSessionMock(AsyncMock):
    # For convenience return a magic mock for any url
    # this can also be done with chained calls
    # The return value can be configured with get_preconfigured_session_mock
    urls = MagicMock()


def get_preconfigured_session_mock(config=None) -> Callable:
    """
    Mock a kelvin session with chained calls to do e.g. self.session.post()
    fast-api overrides have to be callables, so we simply return a
    >>> callable_mock = get_preconfigured_session_mock({'post.return_value': "someobject"})})
    >>> callable_mock()

    :param config:
    :return: callable kelvin session mock
    """
    if config is None:
        config = {}
    my_mock = KelvinSessionMock()
    my_mock.configure_mock(**config)

    async def call_me():
        return my_mock

    return call_me
